@@ -299,22 +299,8 @@ void ssl_init_Module(server_rec *s, pool *p)
      *  initialize servers
      */
     ssl_log(s, SSL_LOG_INFO, "Init: Initializing (virtual) servers for SSL");
-    for (; s != NULL; s = s->next) {
-        sc = mySrvConfig(s);
-
-        /*
-         * Give out warnings when HTTPS is configured for
-         * the HTTP port or vice versa
-         */
-        if (sc->bEnabled && s->port == DEFAULT_HTTP_PORT)
-            ssl_log(s, SSL_LOG_WARN,
-                    "Init: You configured HTTPS(%d) on the standard HTTP(%d) port!",
-                    DEFAULT_HTTPS_PORT, DEFAULT_HTTP_PORT);
-        if (!sc->bEnabled && s->port == DEFAULT_HTTPS_PORT)
-            ssl_log(s, SSL_LOG_WARN,
-                    "Init: You configured HTTP(%d) on the standard HTTPS(%d) port!",
-                    DEFAULT_HTTP_PORT, DEFAULT_HTTPS_PORT);
-
+    for (s2 = s; s2 != NULL; s2 = s2->next) {
+        sc = mySrvConfig(s2);
         /*
          * Either now skip this server when SSL is disabled for
          * it or give out some information about what we're
@@ -322,16 +308,21 @@ void ssl_init_Module(server_rec *s, pool *p)
          */
         if (!sc->bEnabled)
             continue;
-        ssl_log(s, SSL_LOG_INFO,
+        ssl_log(s2, SSL_LOG_INFO,
                 "Init: Configuring server %s for SSL protocol",
-                ssl_util_vhostid(p, s));
+                ssl_util_vhostid(p, s2));
 
         /*
          * Read the server certificate and key
          */
-        ssl_init_ConfigureServer(s, p, sc);
+        ssl_init_ConfigureServer(s2, p, sc);
     }
 
+    /*
+     * Configuration consistency checks
+     */
+    ssl_init_CheckServers(s, p);
+
     /*
      *  Announce mod_ssl and SSL library in HTTP Server field
      *  as ``mod_ssl/X.X.X OpenSSL/X.X.X''
@@ -591,6 +582,14 @@ void ssl_init_ConfigureServer(server_rec *s, pool *p, SSLSrvConfigRec *sc)
                         "(BasicConstraints: pathlen == %d > 0 !?)",
                         cpVHostID, (i == SSL_AIDX_RSA ? "RSA" : "DSA"), pathlen);
             }
+            if (SSL_X509_getCN(p, sc->pPublicCert[i], &cp)) {
+                if (strNE(s->server_hostname, cp)) {
+                    ssl_log(s, SSL_LOG_WARN,
+                        "Init: (%s) %s server certificate CommonName (CN) `%s' "
+                        "does NOT match server name!?", cpVHostID, 
+                        (i == SSL_AIDX_RSA ? "RSA" : "DSA"), cp);
+                }
+            }
         }
     }
 
@@ -683,6 +682,71 @@ void ssl_init_ConfigureServer(server_rec *s, pool *p, SSLSrvConfigRec *sc)
     return;
 }
 
+void ssl_init_CheckServers(server_rec *sm, pool *p)
+{
+    server_rec *s;
+    server_rec **ps;
+    SSLSrvConfigRec *sc;
+    ssl_ds_table *t;
+    pool *sp;
+    char *key;
+    BOOL bConflict;
+
+    /*
+     * Give out warnings when a server has HTTPS configured 
+     * for the HTTP port or vice versa
+     */
+    for (s = sm; s != NULL; s = s->next) {
+        sc = mySrvConfig(s);
+        if (sc->bEnabled && s->port == DEFAULT_HTTP_PORT)
+            ssl_log(sm, SSL_LOG_WARN,
+                    "Init: (%s) You configured HTTPS(%d) on the standard HTTP(%d) port!",
+                    ssl_util_vhostid(p, s), DEFAULT_HTTPS_PORT, DEFAULT_HTTP_PORT);
+        if (!sc->bEnabled && s->port == DEFAULT_HTTPS_PORT)
+            ssl_log(sm, SSL_LOG_WARN,
+                    "Init: (%s) You configured HTTP(%d) on the standard HTTPS(%d) port!",
+                    ssl_util_vhostid(p, s), DEFAULT_HTTP_PORT, DEFAULT_HTTPS_PORT);
+    }
+
+    /*
+     * Give out warnings when more than one SSL-aware virtual server uses the
+     * same IP:port. This doesn't work because mod_ssl then will always use
+     * just the certificate/keys of one virtual host (which one cannot be said
+     * easily - but that doesn't matter here).
+     */
+    sp = ap_make_sub_pool(p);
+    t = ssl_ds_table_make(sp, sizeof(server_rec *));
+    bConflict = FALSE;
+    for (s = sm; s != NULL; s = s->next) {
+        sc = mySrvConfig(s);
+        if (!sc->bEnabled)
+            continue;
+        key = ap_psprintf(sp, "%pA:%u", &s->addrs->host_addr, s->addrs->host_port);
+        ps = ssl_ds_table_get(t, key);
+        if (ps != NULL) {
+            ssl_log(sm, SSL_LOG_WARN,
+                    "Init: SSL server IP/port conflict: %s (%s:%d) vs. %s (%s:%d)",
+                    ssl_util_vhostid(p, s), 
+                    (s->defn_name != NULL ? s->defn_name : "unknown"),
+                    s->defn_line_number,
+                    ssl_util_vhostid(p, *ps),
+                    ((*ps)->defn_name != NULL ? (*ps)->defn_name : "unknown"), 
+                    (*ps)->defn_line_number);
+            bConflict = TRUE;
+            continue;
+        }
+        ps = ssl_ds_table_push(t, key);
+        *ps = s;
+    }
+    ssl_ds_table_kill(t);
+    ap_destroy_pool(sp);
+    if (bConflict)
+        ssl_log(sm, SSL_LOG_WARN,
+                "Init: You cannot use name-based virtual hosts in conjunction with SSL!!");
+
+    return;
+}
+
 static int ssl_init_FindCAList_X509NameCmp(X509_NAME **a, X509_NAME **b)
 {
     return(X509_NAME_cmp(*a, *b));
