@@ -185,7 +185,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
         return;
     }
     SSL_clear(ssl);
-    SSL_set_session_id_context(ssl, cpVHostID, strlen(cpVHostID));
+    SSL_set_session_id_context(ssl, (unsigned char *)cpVHostID, strlen(cpVHostID));
     SSL_set_app_data(ssl, conn);
     apctx = ap_ctx_new(conn->pool);
     ap_ctx_set(apctx, "ssl::request_rec", NULL);
